@@ -9,9 +9,7 @@ Created on 2018-07-26 11:40:28
 """
 
 import datetime
-import os
-import re
-import time
+import re as _re
 from urllib.parse import urlparse, urlunparse, urljoin
 
 from espider.utils.tools import search
@@ -30,7 +28,7 @@ SPECIAL_CHARACTERS = [
 ]
 
 SPECIAL_CHARACTER_PATTERNS = [
-    re.compile(special_character) for special_character in SPECIAL_CHARACTERS
+    _re.compile(special_character) for special_character in SPECIAL_CHARACTERS
 ]
 
 
@@ -194,7 +192,7 @@ class Response(res):
                 # return re.sub(regex, r'\1{}\3'.format(absolute_link), html) # 使用正则替换，个别字符不支持。如该网址源代码http://permit.mep.gov.cn/permitExt/syssb/xxgk/xxgk!showImage.action?dataid=0b092f8115ff45c5a50947cdea537726
                 return text.group(1) + absolute_link + text.group(3)
 
-            text = re.sub(regex, replace_href, text, flags=re.S)
+            text = _re.sub(regex, replace_href, text, flags=_re.S)
 
         return text
 
@@ -207,41 +205,11 @@ class Response(res):
 
         return text
 
-    @property
-    def __text(self):
-        """Content of the response, in unicode.
-
-        If Response.encoding is None, encoding will be guessed using
-        ``chardet``.
-
-        The encoding of the response content is determined based solely on HTTP
-        headers, following RFC 2616 to the letter. If you can take advantage of
-        non-HTTP knowledge to make a better guess at the encoding, you should
-        set ``r.encoding`` appropriately before accessing this property.
-        """
-
-        if not self.content:
-            return ""
-
-        # Decode unicode from given encoding.
-        try:
-            content = str(self.content, self.encoding, errors=self.encoding_errors)
-        except (LookupError, TypeError):
-            # A LookupError is raised if the encoding was not found which could
-            # indicate a misspelling or similar mistake.
-            #
-            # A TypeError can be raised if encoding is None
-            #
-            # So we try blindly encoding.
-            content = str(self.content, errors=self.encoding_errors)
-
-        return content
-
     @property
     def text(self):
         if self._cached_text is None:
             if self.encoding and self.encoding.upper() != FAIL_ENCODING:
-                self._cached_text = self.__text
+                self._cached_text = super(Response, self).text
             else:
                 self._cached_text = self._get_unicode_html(self.content)
 
@@ -280,11 +248,17 @@ class Response(res):
     def extract(self):
         return self.selector.get()
 
-    def xpath(self, query, **kwargs):
-        return self.selector.xpath(query, **kwargs).extract()
+    def xpath(self, query, extract=True, **kwargs):
+        if extract:
+            return self.selector.xpath(query, **kwargs).extract()
+        else:
+            return self.selector.xpath(query, **kwargs)
 
-    def xpath_first(self, query, **kwargs):
-        return self.selector.xpath(query, **kwargs).extract_first()
+    def xpath_first(self, query, extract=True, **kwargs):
+        if extract:
+            return self.selector.xpath(query, **kwargs).extract_first()
+        else:
+            return self.selector.xpath(query, **kwargs)
 
     def xpath_map(self, map, **kwargs):
         return self._query_from_map(self.xpath, map, **kwargs)
@@ -292,31 +266,38 @@ class Response(res):
     def xpath_first_map(self, map, **kwargs):
         return self._query_from_map(self.xpath_first, map, **kwargs)
 
-    def css(self, query):
-        return self.selector.css(query).extract()
+    def css(self, query, extract=True):
+        if extract:
+            return self.selector.css(query).extract()
+        else:
+            return self.selector.css(query)
 
-    def css_first(self, query):
-        return self.selector.css(query).extract_first()
+    def css_first(self, query, extract=True):
+        if extract:
+            return self.selector.css(query).extract_first()
+        else:
+            return self.selector.css(query)
 
-    def css_map(self, map):
-        return self._query_from_map(self.css, map)
+    def css_map(self, map, **kwargs):
+        return self._query_from_map(self.css, map, **kwargs)
 
-    def css_first_map(self, map):
-        return self._query_from_map(self.css_first, map)
+    def css_first_map(self, map, **kwargs):
+        return self._query_from_map(self.css_first, map, **kwargs)
 
     def find(self, key, data=None, target_type=None):
         return search(key=key, data=data or self.json, target_type=target_type)
 
-    def find_map(self, map, data=None, target_type=None):
-        return self._query_from_map(self.find, map, data=data, target_type=target_type)
+    def find_map(self, map, data=None, target_type=None, **kwargs):
+        return self._query_from_map(self.find, map, data=data, target_type=target_type, **kwargs)
 
-    def re(self, regex, replace_entities=False):
+    def re(self, regex, replace_entities=False, flags=_re.S):
         """
         @summary: 正则匹配
         注意：网页源码<a class='page-numbers'...  会被处理成<a class="page-numbers" ； 写正则时要写<a class="(.*?)"。 但不会改非html的文本引号格式
         为了使用方便，正则单双引号自动处理为不敏感
         ---------
         @param regex: 正则或者re.compile
+        @param flags: re.S ...
         @param replace_entities: 为True时 去掉&nbsp;等字符， 转义&quot;为 " 等， 会使网页结构发生变化。如在网页源码中提取json， 建议设置成False
         ---------
         @result: 列表
@@ -324,14 +305,14 @@ class Response(res):
 
         # 将单双引号设置为不敏感
         if isinstance(regex, str):
-            regex = re.sub("['\"]", "['\"]", regex)
+            regex = _re.sub("['\"]", "['\"]", regex)
 
-        return self.selector.re(regex, replace_entities)
+        return self.selector.re(regex, replace_entities, flags=flags)
 
-    def re_map(self, map, replace_entities=False):
-        return self._query_from_map(self.re, map, replace_entities=replace_entities)
+    def re_map(self, map, replace_entities=False, **kwargs):
+        return self._query_from_map(self.re, map, replace_entities=replace_entities, **kwargs)
 
-    def re_first(self, regex, default=None, replace_entities=False):
+    def re_first(self, regex, default=None, replace_entities=False, flags=_re.S):
         """
         @summary: 正则匹配
         注意：网页源码<a class='page-numbers'...  会被处理成<a class="page-numbers" ； 写正则时要写<a class="(.*?)"。 但不会改非html的文本引号格式
@@ -339,6 +320,7 @@ class Response(res):
         ---------
         @param regex: 正则或者re.compile
         @param default: 未匹配到， 默认值
+        @param flags: re.S ...
         @param replace_entities: 为True时 去掉&nbsp;等字符， 转义&quot;为 " 等， 会使网页结构发生变化。如在网页源码中提取json， 建议设置成False
         ---------
         @result: 第一个值或默认值
@@ -346,34 +328,50 @@ class Response(res):
 
         # 将单双引号设置为不敏感
         if isinstance(regex, str):
-            regex = re.sub("['\"]", "['\"]", regex)
+            regex = _re.sub("['\"]", "['\"]", regex)
 
-        return self.selector.re_first(regex, default, replace_entities)
+        return self.selector.re_first(regex, default, replace_entities, flags=flags)
 
-    def re_first_map(self, map, default=None, replace_entities=False):
-        return self._query_from_map(self.re_first, map, default=default, replace_entities=replace_entities)
+    def re_first_map(self, map, default=None, replace_entities=False, **kwargs):
+        return self._query_from_map(self.re_first, map, default=default, replace_entities=replace_entities, **kwargs)
 
     def _query_from_map(self, func, map: dict, *args, **kwargs):
         data = {}
         for key, value in map.items():
             if isinstance(value, str):
-                data[key] = func(value, *args, **kwargs)
+                data[key] = self.__get_query_value(func, value, *args, **kwargs)
             elif isinstance(value, dict):
                 data[key] = self._query_from_map(func, value, *args, **kwargs)
             else:
                 print(f'Warning ... query not support {type(value)}')
         return data
 
-    def __del__(self):
-        self.close()
+    def __get_query_value(self, func, query, *args, **kwargs):
+        queries = query.split('||')
+        for qu in queries:
+            if '&&' in qu:
+                delimiter = kwargs.pop('delimiter')
+                value_list = []
+                for _ in qu.split('&&'):
+                    v = func(_.strip(), *args, **kwargs)
+                    if v: value_list.append(v)
+                value = delimiter.join(value_list) if delimiter else value_list
+            else:
+                value = func(qu, *args, **kwargs)
 
-    def open(self, delete_temp_file=False):
-        with open("temp.html", "w", encoding=self.encoding, errors="replace") as html:
+            if value: return value
+
+        else:
+            return ''
+
+    def save_html(self, path=None):
+        with open(path or "index.html", "w", encoding=self.encoding, errors="replace") as html:
             self.encoding_errors = "replace"
             html.write(self.text)
 
-        os.system("open temp.html")
+    def save_content(self, path):
+        with open(path, "wb") as html:
+            html.write(self.content)
 
-        if delete_temp_file:
-            time.sleep(1)
-            os.remove("temp.html")
+    def __del__(self):
+        self.close()
