@@ -1,15 +1,16 @@
+import threading
+import time
 from collections.abc import Generator
-
 from espider.default_settings import *
 from espider.network import Request, Downloader
 import random
 from requests.cookies import cookiejar_from_dict, merge_cookies
-from requests.sessions import Session
 from espider.parser.response import Response
-
+from espider.pipelines import BasePipeline
 from espider.settings import Setting
+from espider.utils import requests
 from espider.utils.tools import url_to_dict, body_to_dict, json_to_dict, headers_to_dict, cookies_to_dict, dict_to_body, \
-    dict_to_json, update
+    dict_to_json, update, search, delete, strip, replace, random_list, human_time
 
 
 class Spider(object):
@@ -54,7 +55,8 @@ class Spider(object):
                 self.request_kwargs[key] = value
 
         self.use_session = use_session
-        self.session = Session()
+        self.session = requests.Session()
+        self.session.headers = self.spider.get('headers')
 
         # 加载 setting
         self.setting = Setting()
@@ -64,12 +66,42 @@ class Spider(object):
         self.downloader_setting = self.setting.get('downloader') if self.setting.get('downloader') else {}
         self.request_setting = self.setting.get('request') if self.setting.get('request') else {}
 
+        # 过滤器
+        self.item_filter = []
+
+        # 插件
+        self.response_extensions = []
+        self.request_extensions = []
+
         self.downloader = Downloader(
             **self.downloader_setting,
-            item_callback=self.item_pipeline,
+            pipeline=BasePipeline(),
             end_callback=self.end,
+            item_filter=self.item_filter,
         )
 
+        # 连接数据库
+        self.db = None
+        self.cursor = None
+
+        # 额外参数
+        self.pocket = {}
+
+        # 数据处理工具
+        self.search = search
+        self.update = update
+        self.replace = replace
+        self.delete = delete
+        self.strip = strip
+        self.random_list = random_list
+
+        # 网络
+        self.get = requests.get
+        self.post = requests.post
+
+        # 时间计算
+        self.start_time = time.time()
+
     def _init_header(self):
         if self.method == 'POST':
             content_type = 'application/x-www-form-urlencoded; charset=UTF-8'
@@ -148,7 +180,7 @@ class Spider(object):
     def cookie_jar(self):
         return self.spider.get('cookies')
 
-    def update(self, **kwargs):
+    def update_spider(self, **kwargs):
         self.spider = update({key: value for key, value in kwargs.items()}, data=self.spider)
 
     def update_cookie_from_header(self):
@@ -162,11 +194,14 @@ class Spider(object):
             self.spider['cookies'] = merge_cookies(self.spider.get('cookies'), response.cookies)
 
     def request(self, url=None, method=None, data=None, json=None, headers=None, cookies=None, callback=None,
-                error_callback=None, failed_callback=None, retry_callback=None, args=None, priority=None,
-                use_session=None, **kwargs):
+                args=None, priority=None, use_session=None, **kwargs):
 
         if use_session is None: use_session = self.use_session
 
+        if isinstance(headers, str): headers = headers_to_dict(headers)
+        if isinstance(cookies, str): cookies = cookies_to_dict(cookies)
+        if isinstance(json, str): json = json_to_dict(json)
+
         request_kwargs = {
             **self.request_kwargs,
             'url': url or self.url,
@@ -177,9 +212,6 @@ class Spider(object):
             'cookies': cookies,
             'priority': priority,
             'callback': callback or self.parse,
-            'retry_callback': retry_callback or self.retry_pipeline,
-            'failed_callback': failed_callback or self.failed_pipeline,
-            'error_callback': error_callback or self.error_pipeline,
             'downloader': self.downloader,
             'args': args,
             'session': self.session if use_session else None,
@@ -188,9 +220,8 @@ class Spider(object):
         }
         return Request(**request_kwargs)
 
-    def form_request(self, url=None, data=None, json=None, headers=None, cookies=None, callback=None,
-                     error_callback=None, failed_callback=None, retry_callback=None, args=None, priority=None,
-                     use_session=False, **kwargs):
+    def form_request(self, url=None, data=None, json=None, headers=None, cookies=None, callback=None, args=None,
+                     priority=None, use_session=False, **kwargs):
 
         return self.request(
             self,
@@ -201,9 +232,6 @@ class Spider(object):
             headers=headers,
             cookies=cookies,
             callback=callback,
-            retry_callback=retry_callback,
-            failed_callback=failed_callback,
-            error_callback=error_callback,
             args=args,
             priority=priority,
             use_session=use_session,
@@ -214,41 +242,59 @@ class Spider(object):
         isinstance(response, Response)
         return self.request(**response.request_kwargs)
 
+    def before(self):
+        pass
+
     def start_requests(self):
         """
-        起点
+        入口
         """
         yield ...
 
     def run(self):
+        self.before()
+
         if type(self.downloader).__name__ == 'type':
             self.downloader = self.downloader()
 
-        request_generator = self.start_requests()
-        assert isinstance(request_generator, Generator), 'function start_requests must be a generator'
+        assert isinstance(self.start_requests(), Generator), 'function start_requests must be a generator'
+
+        spider_thread = threading.Thread(target=self._run)
+        spider_thread.start()
+        self.downloader.start()
+        spider_thread.join()
+
+    def _run(self):
         for request in self.start_requests():
             self.downloader.push(request)
 
-        self.downloader.start()
-
     def parse(self, response, *args, **kwargs):
         pass
 
-    def retry_pipeline(self, request, *args, **kwargs):
-        print(f'[{kwargs.get("status_code")}] Retry-{request.retry_count}: {request.request_kwargs}')
-        return request
-
-    def error_pipeline(self, exception, *args, **kwargs):
-        print(f'Request Error ... {exception}, args: {args}, kwargs: {kwargs}')
-
-    def failed_pipeline(self, response, *args, **kwargs):
-        print(f'Request Failed ... {response} Retry: {response.retry_times} args: {args} kwargs: {kwargs}')
-
-    def item_pipeline(self, item, *args, **kwargs):
-        pass
-
     def end(self):
-        pass
+        cost_time = human_time(time.time() - self.start_time)
+        print('Time: {} day {} hour {} minute {:.3f} second'.format(*cost_time))
+
+    def _add_extension(self, extension, target=None, *args, **kwargs):
+        if type(extension).__name__ == 'type':
+            extension = extension()
+
+        new_extension = {
+            'extension': extension,
+            'args': args,
+            'kwargs': kwargs,
+            'count': 0
+        }
+        if target == 'request':
+            self.request_extensions.append(new_extension)
+        elif target == 'response':
+            self.response_extensions.append(new_extension)
+
+    def add_request_extension(self, extension, *args, **kwargs):
+        self._add_extension(extension, target='request', *args, **kwargs)
+
+    def add_response_extension(self, extension, *args, **kwargs):
+        self._add_extension(extension, target='response', *args, **kwargs)
 
     def __repr__(self):
         msg = f'{self.__name__}({self.method}, url=\'{self.url}\', body=\'{self.body or self.json}\', headers={self.headers}, cookies={self.cookies})'
